﻿Imports System.Runtime.InteropServices

Public Class LVSort

    <StructLayout(LayoutKind.Sequential)> _
Public Structure HDITEM

        Public mask As Int32
        Public cxy As Int32
        <MarshalAs(UnmanagedType.LPTStr)> _
        Public pszText As String
        Public hbm As IntPtr
        Public cchTextMax As Int32
        Public fmt As Int32
        Public lParam As Int32
        Public iImage As Int32
        Public iOrder As Int32
    End Structure

    ''// Parameters for ListView-Headers
    Private Const HDI_FORMAT As Int32 = &H4
    Private Const HDF_LEFT As Int32 = &H0
    Private Const HDF_STRING As Int32 = &H4000
    Private Const HDF_SORTUP As Int32 = &H400
    Private Const HDF_SORTDOWN As Int32 = &H200
    Private Const LVM_GETHEADER As Int32 = &H1000 + 31  '' LVM_FIRST + 31
    Private Const HDM_GETITEM As Int32 = &H1200 + 11  '' // HDM_FIRST + 11
    Private Const HDM_SETITEM As Int32 = &H1200 + 12  '' HDM_FIRST + 12

    'There are two ways to "rename" a call to the Windows API, one way is to use the EntryPoint attribute, like so...

    'GetHeaderHnd function
    <DllImport("USER32.DLL", EntryPoint:="SendMessage")> _
    Public Shared Function GetHeaderHnd(ByVal hWnd As IntPtr, ByVal Msg As Integer, ByVal wParam As IntPtr, ByVal lParam As IntPtr) As IntPtr
    End Function

    ' Declare Function SendMessage Lib "user32.dll" Alias "SendMessageA" (ByVal hWnd As IntPtr, ByVal Msg As Integer, ByVal wParam As IntPtr, ByVal lParam As IntPtr) As IntPtr
    'Private Delegate Function GetHeaderDelegate(ByVal hWnd As IntPtr, ByVal Msg As Integer, ByVal wParam As IntPtr, ByVal lParam As IntPtr) As IntPtr
    'Shared GetHeaderHnd As GetHeaderDelegate = AddressOf SendMessage

    '... and the other way is to use delegates like this

    'Get/Set HDItem
    Declare Function SendMessage Lib "user32.dll" Alias "SendMessageA" (ByVal Handle As IntPtr, ByVal msg As Int32, ByVal wParam As IntPtr, ByRef lParam As HDITEM) As IntPtr
    Private Delegate Function GetSetHDITEMDelegate(ByVal Handle As IntPtr, ByVal msg As Int32, ByVal wParam As IntPtr, ByRef lParam As HDITEM) As IntPtr
    Shared SendMessageITEM As GetSetHDITEMDelegate = AddressOf SendMessage


    Public Shared Sub SetSortIcons(ByVal lstView As ListView, ByRef previouslySortedColumn As Integer, ByVal newSortColumn As Integer)

        Dim hHeader As IntPtr = GetHeaderHnd(CType(lstView.Handle, IntPtr), LVM_GETHEADER, IntPtr.Zero, IntPtr.Zero)

        Dim newColumn As IntPtr = New IntPtr(newSortColumn)
        Dim prevColumn As IntPtr = New IntPtr(previouslySortedColumn)
        Dim HDITEM As HDITEM

        Dim rtn As IntPtr


        '' Only update the previous item if it existed and if it was a different one.
        If (previouslySortedColumn <> -1 AndAlso previouslySortedColumn <> newSortColumn) Then

            '' Clear icon from the previous column.
            HDITEM = New HDITEM()
            HDITEM.mask = HDI_FORMAT
            rtn = SendMessageITEM(hHeader, HDM_GETITEM, prevColumn, HDITEM)
            HDITEM.fmt = HDITEM.fmt And Not HDF_SORTDOWN And Not HDF_SORTUP
            rtn = SendMessageITEM(hHeader, HDM_SETITEM, prevColumn, HDITEM)
        End If

        '' Set icon on the new column.
        HDITEM = New HDITEM()
        HDITEM.mask = HDI_FORMAT
        rtn = SendMessageITEM(hHeader, HDM_GETITEM, newColumn, HDITEM)
        If (lstView.Sorting = SortOrder.Ascending) Then
            HDITEM.fmt = HDITEM.fmt And Not HDF_SORTDOWN
            HDITEM.fmt = HDITEM.fmt Or HDF_SORTUP
        Else
            HDITEM.fmt = HDITEM.fmt And Not HDF_SORTUP
            HDITEM.fmt = HDITEM.fmt Or HDF_SORTDOWN
        End If
        rtn = SendMessageITEM(hHeader, HDM_SETITEM, newColumn, HDITEM)
        previouslySortedColumn = newSortColumn
    End Sub


End Class


' Implements a type-aware comparer for ListView columns.
Class ListViewComparer
    Implements IComparer

    Private m_sortColumn As Integer
    Public Property SortColumn() As Integer
        Get
            Return m_sortColumn
        End Get
        Set(ByVal value As Integer)
            m_sortColumn = value
        End Set
    End Property

    Public Sub New()
    End Sub


    Public Function Compare(ByVal objItem1 As Object, ByVal objItem2 As Object) As Integer _
        Implements System.Collections.IComparer.Compare

        Dim lvItem1 As ListViewItem = DirectCast(objItem1, ListViewItem)
        Dim lvItem2 As ListViewItem = DirectCast(objItem2, ListViewItem)

        Dim sortType As String = String.Empty
        If lvItem1.ListView.Columns(SortColumn).Tag IsNot Nothing AndAlso lvItem2.ListView.Columns(SortColumn).Tag IsNot Nothing Then
            If lvItem1.ListView.Columns(SortColumn).Tag.ToString = lvItem2.ListView.Columns(SortColumn).Tag.ToString Then
                sortType = lvItem1.ListView.Columns(SortColumn).Tag.ToString
            End If
        End If

        ' get column value as string (parse to correct type later)
        Dim strItem1 As String = String.Empty
        If lvItem1.SubItems.Count > SortColumn Then
            strItem1 = lvItem1.SubItems(SortColumn).Text
        End If

        Dim strItem2 As String = String.Empty
        If lvItem2.SubItems.Count > SortColumn Then
            strItem2 = lvItem2.SubItems(SortColumn).Text
        End If

        Dim lvSortOrder As SortOrder = lvItem1.ListView.Sorting

        Dim ascDesc As Integer = If(lvSortOrder = SortOrder.Ascending, 1, -1)
        Select Case LCase(sortType)
            Case "string"
                Return String.Compare(strItem1, strItem2) * ascDesc
            Case "number"
                Dim intItem1 As Integer
                Dim intItem2 As Integer
                If Integer.TryParse(strItem1, intItem1) Then
                    If Integer.TryParse(strItem2, intItem2) Then
                        Return intItem1.CompareTo(intItem2) * ascDesc
                    End If
                End If

            Case "date"
                Dim dtItem1 As DateTime
                Dim dtItem2 As DateTime
                If DateTime.TryParse(strItem1, dtItem1) Then
                    If DateTime.TryParse(strItem2, dtItem2) Then
                        Return dtItem1.CompareTo(dtItem2) * ascDesc
                    End If
                End If
        End Select

        Return String.Compare(strItem1, strItem2) * ascDesc


    End Function
End Class
